/****************************************************  Product Functions  *****************************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following functions are used to initialize the hardware specific to each SigFPGA model.

	Rev History: 12-08-2014:	Added Model 371 support.
				 10-20-2014:	Inital release.

	Copyright 2014, Red Rapids, Inc.

****************************************************  Product Functions  *****************************************************/

#include <stdlib.h>

#include "adapter.h"
#include "sigfpga.h"
#include "dma_functions.h"
#include "common_functions.h"
#include "channel_functions.h"
#include "device_functions.h"
#include "product_functions.h"


// Intitialize the Model 371 hardware.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long M371Initialize(s_Adapter *p_HW, s_M371Devices M371Devices, s_Channel *p_RXChannel)
{
	unsigned long Status = 0;
	size_t cnt = 0;
	s_User *User = NULL;

	// Cast the user defined pointer from the hardware handle.
	User = (s_User *)(p_HW->User);

	// The Model 371 is equipped with two receiver channels and no transmitter channels.
	User->RXChannelCount = ChannelCount(p_HW, 0);
	User->TXChannelCount = ChannelCount(p_HW, 1);
	// The Model 371 RX channels produce a 16-bit data item size.
	User->RXDataItemSize = 16;
	User->TXDataItemSize = 0;

	// Configure the I/O signal direction and impedance of the command/status port expander.
	// Note: The 100 MHz configuration flash clock is disabled.
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x3, 0xF8);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x7, 0x07);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x5, 0x00);

	// Configure the dual ADC that is assigned to RX channels #1 and #2 by loading the default values.
	for (cnt = 0; cnt < 21; cnt++)
		Status += ADS42LB69Write(p_HW, M371_RX1TO2_ADC, M371Devices.RX1TO2_ADC.Payload[cnt][0], M371Devices.RX1TO2_ADC.Payload[cnt][1]);

	// Configure the trim DACs from the default structure values.
	// Note: Trim DACs are only present on DC coupled units.
	for (cnt = 0; cnt < 3; cnt++)
	{
		Status += LTC1661Write(p_HW, M371_RX1_TRIM_DAC, M371Devices.RX1_TRIM_DAC.Payload[cnt][0], M371Devices.RX1_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M371_RX2_TRIM_DAC, M371Devices.RX2_TRIM_DAC.Payload[cnt][0], M371Devices.RX2_TRIM_DAC.Payload[cnt][1]);
	}

	// Configure the amplifier enables from the default structure values.
	// Note: Amplifiers are only present on DC coupled units.
	for (cnt = 0; cnt < 2; cnt++)
		Status += ADL5566Write(p_HW, CMD_STAT1, M371Devices.RX1TO2_AMP_ENBL.Enable[cnt][0], M371Devices.RX1TO2_AMP_ENBL.Enable[cnt][1]);

	// Determine whether the external sample clock is active.
	if (ClockStatus(p_HW, CMD_STAT1) < 2)
		// An external clock was detected, so change the selected input to the clock distribution chip.
		// Note: The default input is the on-board synthesizer.
		M371Devices.ClockDist.Payload[9][1] = 0x02;

	// Hardware reset the clock distribution chip.
	Status += AD9512Write(p_HW, M371_CLK_DIST, 0x0000, 0x30);
	Status += AD9512Write(p_HW, M371_CLK_DIST, 0x005A, 0x01);
	Status += AD9512Write(p_HW, M371_CLK_DIST, 0x0000, 0x10);
	Status += AD9512Write(p_HW, M371_CLK_DIST, 0x005A, 0x01);
	// Configure the clock distribution chip by loading the default values.
	for (cnt = 0; cnt < 22; cnt++)
		Status += AD9512Write(p_HW, M371_CLK_DIST, M371Devices.ClockDist.Payload[cnt][0], M371Devices.ClockDist.Payload[cnt][1]);

	return(Status);
}

// Intitialize the Model 376 hardware.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long M376Initialize(s_Adapter *p_HW, s_M376Devices M376Devices, s_Channel *p_RXChannel)
{
	unsigned long Status = 0;
	size_t cnt = 0;
	s_User *User = NULL;

	// Cast the user defined pointer from the hardware handle.
	User = (s_User *)(p_HW->User);

	// The standard Model 376 has two receiver channels and no transmitter channels.
	User->RXChannelCount = ChannelCount(p_HW, 0);
	User->TXChannelCount = ChannelCount(p_HW, 1);
	// The SigFPGA hardware sign extends the 12-bit sample data to a 16-bit data item size.
	User->RXDataItemSize = 16;
	User->TXDataItemSize = 0;

	// Configure the I/O signal direction and impedance of the command/status port expander.
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x3, 0x38);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x7, 0xC7);

	// Configure the ADC from the default structure values.
	for (cnt = 0; cnt < 11; cnt++)
		Status += ADC12D1600Write(p_HW, M376_RX1TO2_ADC, M376Devices.RX1TO2_ADC.Payload[cnt][0], M376Devices.RX1TO2_ADC.Payload[cnt][1]);

	// Configure the trim DACs from the default structure values.
	// Note: Trim DACs are only present on DC coupled units.
	for (cnt = 0; cnt < 3; cnt++)
	{
		Status += LTC1661Write(p_HW, M376_RX1_TRIM_DAC, M376Devices.RX1_TRIM_DAC.Payload[cnt][0], M376Devices.RX1_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M376_RX2_TRIM_DAC, M376Devices.RX2_TRIM_DAC.Payload[cnt][0], M376Devices.RX2_TRIM_DAC.Payload[cnt][1]);
	}

	// Configure the amplifier enables from the default structure values.
	// Note: Amplifiers are only present on DC coupled units.
	for (cnt = 0; cnt < 2; cnt++)
		Status += ADL5566Write(p_HW, CMD_STAT1, M376Devices.RX1TO2_AMP_ENBL.Enable[cnt][0], M376Devices.RX1TO2_AMP_ENBL.Enable[cnt][1]);

	return(Status);
}

// Intitialize the Model 377 hardware.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long M377Initialize(s_Adapter *p_HW, s_M377Devices M377Devices, s_Channel *p_RXChannel)
{
	unsigned long Status = 0;
	size_t cnt = 0;
	s_User *User = NULL;

	// Cast the user defined pointer from the hardware handle.
	User = (s_User *)(p_HW->User);

	// The Model 377 can is equipped with four receiver channels and no transmitter channels.
	User->RXChannelCount = ChannelCount(p_HW, 0);
	User->TXChannelCount = ChannelCount(p_HW, 1);
	// The Model 377 RX channels produce a 16-bit data item size.
	User->RXDataItemSize = 16;
	User->TXDataItemSize = 0;

	// Configure the I/O signal direction and impedance of the command/status port expander.
	// Note: The 100 MHz configuration flash clock is disabled.
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x3, 0xF8);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x7, 0x07);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x5, 0x00);

	// Configure the first ADC that is assigned to RX channels #1 and #2 by loading the default values.
	for (cnt = 0; cnt < 21; cnt++)
		Status += ADS42LB69Write(p_HW, M377_RX1TO2_ADC, M377Devices.RX1TO2_ADC.Payload[cnt][0], M377Devices.RX1TO2_ADC.Payload[cnt][1]);

	// Configure the second ADC that is assigned to RX channels #3 and #4 by loading the default values.
	for (cnt = 0; cnt < 21; cnt++)
		Status += ADS42LB69Write(p_HW, M377_RX3TO4_ADC, M377Devices.RX3TO4_ADC.Payload[cnt][0], M377Devices.RX3TO4_ADC.Payload[cnt][1]);

	// Configure the trim DACs from the default structure values.
	// Note: Trim DACs are only present on DC coupled units.
	for (cnt = 0; cnt < 3; cnt++)
	{
		Status += LTC1661Write(p_HW, M377_RX1_TRIM_DAC, M377Devices.RX1_TRIM_DAC.Payload[cnt][0], M377Devices.RX1_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M377_RX2_TRIM_DAC, M377Devices.RX2_TRIM_DAC.Payload[cnt][0], M377Devices.RX2_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M377_RX3_TRIM_DAC, M377Devices.RX3_TRIM_DAC.Payload[cnt][0], M377Devices.RX3_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M377_RX4_TRIM_DAC, M377Devices.RX4_TRIM_DAC.Payload[cnt][0], M377Devices.RX4_TRIM_DAC.Payload[cnt][1]);
	}

	// Configure the amplifier enables from the default structure values.
	// Note: Amplifiers are only present on DC coupled units.
	for (cnt = 0; cnt < 2; cnt++)
	{
		Status += ADL5566Write(p_HW, CMD_STAT1, M377Devices.RX1TO2_AMP_ENBL.Enable[cnt][0], M377Devices.RX1TO2_AMP_ENBL.Enable[cnt][1]);
		Status += ADL5566Write(p_HW, CMD_STAT1, M377Devices.RX3TO4_AMP_ENBL.Enable[cnt][0], M377Devices.RX3TO4_AMP_ENBL.Enable[cnt][1]);
	}

	// Determine whether the external sample clock is active.
	if (ClockStatus(p_HW, CMD_STAT1) < 2)
		// An external clock was detected, so change the selected input to the clock distribution chip.
		// Note: The default input is the on-board synthesizer.
		M377Devices.ClockDist.Payload[9][1] = 0x02;

	// Hardware reset the clock distribution chip.
	Status += AD9512Write(p_HW, M377_CLK_DIST, 0x0000, 0x30);
	Status += AD9512Write(p_HW, M377_CLK_DIST, 0x005A, 0x01);
	Status += AD9512Write(p_HW, M377_CLK_DIST, 0x0000, 0x10);
	Status += AD9512Write(p_HW, M377_CLK_DIST, 0x005A, 0x01);
	// Configure the clock distribution chip by loading the default values.
	for (cnt = 0; cnt < 22; cnt++)
		Status += AD9512Write(p_HW, M377_CLK_DIST, M377Devices.ClockDist.Payload[cnt][0], M377Devices.ClockDist.Payload[cnt][1]);

	return(Status);
}
